<?php
/* Logaholic Web Analytics software             Copyright(c) 2005-2013 Logaholic B.V.
 *                                                               All rights Reserved.
 * This code is subject to the Logaholic license. Unauthorized copying is prohibited.
 * support@logaholic.com                         http://www.logaholic.com/License.txt
*/ 
/**
* @desc This is the master report class. All other reports should be childen of this parent class
* it is assumed that this is included somewhere where common.inc.php is also included 
*/
if(!defined('APP_INCLUDE')){ die('invalid inclusion'); }

include_once(logaholic_dir() . "includes/datavisualization.php");

class DataVisualizationV2 extends DataVisualization{

	/*
	**	Conver Trend Data array to nvd3 Stacked Area array
		- column 0 must be a date it is an x axes
	**	- $sort_key 		=> sort the value on this column
	**	- $column_header 	=> the key of the column header
	**	- $value_key 		=> the key of the value
	*/
	function ConvertToStackedAreaData($data, $column_header, $x_key, $y_key, $sort_key){
		$converted = array();
		$i = 0;
		foreach ($data as $row) {
			$k = $row[$sort_key];
			$converted[$k]['key'] = $row[$column_header];

			# add the values to the array and make sure the value data is an int or else the graph will be blown up
			$converted[$k]['values'][] = array( 
				"label" => $row[$x_key],
				"value" => (int)$row[$y_key]
			); 
			$i ++;
		}
		return $converted;
	}

	/*
	**	Changes a data set from getTrendDataFromFiles to a normal GetReportData data set
	**	Array(
	    [20140824] => Array(
            [0] => Array(
                    [0] => _0_TO_10_SECONDS           
                    [1] => 3668
                    [2] => 5771
                    [3] => 72.59054
                    [4] => 1
                )
            )
        )
	**	To =>
	**	Array(
		[0] => Array(
                [0] => 20140824           
                [1] => _0_TO_10_SECONDS           
                [2] => 3668
                [3] => 5771
                [4] => 72.59054
                [5] => 1
            )
        )
	*/
	function FlattenTrendData($data){
		$flatten = array();
		$r = 0;
		foreach ($data as $key => $rows) {

			foreach ($rows as $row) {
				# Set the first [0] as the key of the first array
				$flatten[$r][0] = $key;
	
				$i = 1;
				foreach ($row as $k => $v) {
					$flatten[$r][$i] = $v;
					
					$i ++;
				}
				$r ++;
			}
		}
		return $flatten;
	}

	function StackedAreaGraph($data, $column_header = 1, $x_key = 0, $y_key = 2, $sort_key = 0){

		# Convert the get trend data array to a d3graph array
		$data = $this->ConvertToStackedAreaData($data, $column_header, $x_key, $y_key, $sort_key);

		# Create the d3 area
		$d3graph_class = "D3" . md5(time());
		echo "<div style='height: 500px;'><svg style='height:100%;' class='$d3graph_class d3graph'></svg></div>";

		?><script>
		var data_set = <?php echo json_encode($data); ?>;
		var colors = d3.scale.category20();
		keyColor = function(d, i) {return colors(d.key)};

		nv.addGraph(function() {
		    var chart = nv.models.stackedAreaChart()
		                  .margin({right: 100})
		                  .x(function(d) { return d.label })   //We can modify the data accessor functions...
		                  .y(function(d) { return d.value })   //...in case your data is formatted differently.
		                  .useInteractiveGuideline(true)    //Tooltips which show all data points. Very nice!
		                  // .transitionDuration(100)
		                  .showControls(false)       //Allow user to choose 'Stacked', 'Stream', 'Expanded' mode.
		                  .clipEdge(true)
		     			  .style('expand')
	               		  .color(keyColor)
	               		  ;

		    //Format x-axis labels with custom function.
		    chart.xAxis
		        .tickFormat(function(d) { 
		        	d = parseInt(d);
		        	return d3.time.format('%x')(new Date(d));
		    });

		    chart.yAxis
		        .tickFormat(d3.format('2f'));

		    d3.select('.<?php echo $d3graph_class; ?>')
		      .datum(data_set)
		      .call(chart);

		    nv.utils.windowResize(chart.update);

		    return chart;
		  });
		</script><?php
	}
}
?>